@@ -12,6 +12,7 @@ import awswrangler as wr
 import pandas as pd
 import inspect
 from inspect import signature
+from urllib.parse import urlparse
 
 logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                     level=logging.INFO)
@@ -630,7 +631,7 @@ class DataPlate:
                             'Bad Access Key! Get your access key at: {}'.format(
                                 self.base_url))
                     if r.status_code == 206:
-                        logging.info('running notebook...')
+                        logging.info('running notebook, processing...')
                         time.sleep(5)
                     raise Error(
                         'Bad HTTP exit status returned from the API: {}. Error was: {}'.
@@ -653,4 +654,206 @@ class DataPlate:
 
         except Exception as e:
             logging.exception('run_notebook , ' + str(e))
-            raise Error(e)
\ No newline at end of file
+            raise Error(e)
+
+
+    def list_runs_notebook(self, n=10, notebook = None, rule = None):
+        """Returns a pandas data frame of the runs, with the most recent at the top.
+        Args:
+        n (int): The number of runs to return or all runs if 0 (default: 10)
+        notebook (str): If not None, return only runs of this notebook (default: None)
+        rule (str): If not None, return only runs invoked by this rule (default: None)
+        """
+
+        try:
+
+            params = {}
+            if n:
+                params['n'] = n
+            if notebook:
+                params['notebook'] = notebook
+            if rule:
+                params['rule'] = rule
+
+            headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
+
+            try:
+                logging.info('sending request...')
+                r = self.session.post( \
+                    '{}/api/aws/listRunsNotebook'.format(self.base_url), params=params, data="",
+                    headers=headers, stream=True, allow_redirects=False)
+
+                if r.status_code != 200:
+                    if r.status_code == 302:
+                        raise Error(
+                            'Bad Access Key! Get your access key at: {}'.format(
+                                self.base_url))
+                    if r.status_code == 206:
+                        logging.info('analysing notebook runs...')
+                        time.sleep(5)
+                    raise Error(
+                        'Bad HTTP exit status returned from the API: {}. Error was: {}'.
+                            format(r.status_code, r.text))
+
+                logging.info('Parsing response')
+                # logging.info(str(r.text))
+                if r.text:
+                    rJson = json.loads(r.text)
+                    logging.info('list notebook runs finished successfully.')
+                    df = pd.read_json(rJson)
+                    df['Created'] = pd.to_datetime(df['Created'], unit='ms',utc=True)
+                    df['Start'] = pd.to_datetime(df['Start'], unit='ms', utc=True)
+                    df['End'] = pd.to_datetime(df['End'], unit='ms', utc=True)
+                    df['Elapsed'] = pd.to_timedelta(df['Elapsed'], unit='ms')
+                    return df
+                else:
+                    logging.exception('Could not find proper output, please check your parameters')
+                # return r.text
+
+            except (requests.exceptions.ConnectionError,
+                    requests.exceptions.ReadTimeout) as e:
+                logging.exception('Got ConnectionError/ReadTimeout exception.')
+                raise Error(e)
+
+        except Exception as e:
+            logging.exception('list_runs_notebook , ' + str(e))
+            raise Error(e)
+
+
+
+    def stop_run_notebook(self, job_name= None):
+        """Stop the named processing job.
+        Args:
+        job_name (string): The name of the job to stop. use list_runs_notebook to get specific notebook Job name
+        """
+
+        try:
+
+            params = {}
+            if job_name:
+                params['jobname'] = job_name
+            else:
+                raise Error('Not a valid job name, use list_runs_notebook function to get specific notebook Job name.')
+
+            headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
+
+            try:
+                logging.info('sending request...')
+                r = self.session.post( \
+                    '{}/api/aws/stopNotebook'.format(self.base_url), params=params, data="",
+                    headers=headers, stream=True, allow_redirects=False)
+
+                if r.status_code != 200:
+                    if r.status_code == 302:
+                        raise Error(
+                            'Bad Access Key! Get your access key at: {}'.format(
+                                self.base_url))
+                    if r.status_code == 206:
+                        logging.info('analysing notebook stops...')
+                        time.sleep(5)
+                    raise Error(
+                        'Bad HTTP exit status returned from the API: {}. Error was: {}'.
+                            format(r.status_code, r.text))
+
+                logging.info('Parsing response')
+                # logging.info(str(r.text))
+                if r.text:
+                    logging.info(r.text)
+                    return
+                else:
+                    logging.exception('Could not find proper result, please check your parameters')
+                # return r.text
+
+            except (requests.exceptions.ConnectionError,
+                    requests.exceptions.ReadTimeout) as e:
+                logging.exception('Got ConnectionError/ReadTimeout exception.')
+                raise Error(e)
+
+        except Exception as e:
+            logging.exception('stop_run_notebook , ' + str(e))
+            raise Error(e)
+
+
+    def download_notebook_result(self, result_s3_file= None, output="."):
+        """Download the output notebook from a previously completed job.
+
+        Args:
+          result_s3_file (str): The name of the SageMaker Processing Job Result that executed the notebook. (Required). use list_runs_notebook to get specific Result of Job
+          output (str): The directory to copy the output file to. (Default: the current working directory)
+
+        Returns:
+          The filename of the downloaded notebook.
+        """
+
+        try:
+            params = {}
+            if result_s3_file:
+                params['result_file'] = result_s3_file
+            else:
+                raise Error('Not a valid job name, use list_runs_notebook function to get specific notebook Job name.')
+
+            headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
+
+            try:
+                logging.info('sending request...')
+                r = self.session.post( \
+                    '{}/api/aws/downloadNotebook'.format(self.base_url), params=params, data="",
+                    headers=headers, stream=True, allow_redirects=False)
+
+                if r.status_code != 200:
+                    if r.status_code == 302:
+                        raise Error(
+                            'Bad Access Key! Get your access key at: {}'.format(
+                                self.base_url))
+                    if r.status_code == 206:
+                        logging.info('analysing notebook stops...')
+                        time.sleep(5)
+                    raise Error(
+                        'Bad HTTP exit status returned from the API: {}. Error was: {}'.
+                            format(r.status_code, r.text))
+
+                logging.info('Parsing response')
+                # logging.info(str(r.text))
+                if r.text:
+                    # rJson = json.loads(r.text)
+                    logging.info('Got notebook result successfully.')
+                    # return json.dumps(rJson)
+                    if not os.path.exists(output):
+                        try:
+                            os.makedirs(output)
+                        except OSError as e:
+                            raise Error(f'Could not crate output directory {output}')
+
+
+                    o = urlparse(result_s3_file, allow_fragments=False)
+                    # ParseResult(scheme='s3', netloc='bucket_name', path='/folder1/folder2/file1.json', params='', query='',
+                    #             fragment='')
+                    base_notebook_name = ""
+                    split_path = o.path.split('/')
+                    if split_path and len(split_path) > 0:
+                        if len(split_path[-1]) > 0 and split_path[-1].find('.') >= 0:
+                            base_notebook_name = split_path[-1]
+
+                    filename_out = '/'.join([str(output.rstrip("/")), str(base_notebook_name)])
+                    # with open(filename_out.rstrip("/"), 'wb') as fh:
+                    #     copyfileobj(json.loads(r.text), fh)
+                    f = open(filename_out.rstrip("/"), 'w', encoding = 'utf-8')
+                    if f:
+                        # writing file
+                        json.dump(json.loads(r.text), f, ensure_ascii=False)#, indent=4)
+                        # Closing file
+                        f.close()
+
+                    logging.info('Done writing to file.')
+                else:
+                    logging.exception('Could not find proper result, please check your parameters')
+                # return r.text
+
+            except (requests.exceptions.ConnectionError,
+                    requests.exceptions.ReadTimeout) as e:
+                logging.exception('Got ConnectionError/ReadTimeout exception.')
+                raise Error(e)
+
+        except Exception as e:
+            logging.exception('download_notebook_result , ' + str(e))
+            raise Error(e)
